require('./helpers').should;

var
  helpers = require('./helpers'),
  debuglog = helpers.debuglog,
  _ = require('lodash'),
  chai = helpers.chai;

chai.use(helpers.cap);


describe('mounts', function () {
  var newVault = helpers.getEmptyVault();
  var myVault;

  before(function () {
    return helpers.getReadyVault().then(function (vault) {
      myVault = vault;
    });

  });

  describe('#getMounts', function () {

    it('should reject with an Error if not initialized or unsealed', function () {
      return newVault.getMounts().should.be.rejectedWith(/Vault has not been initialized/);
    });

    it('should update internal state with list of mounts', function () {
      var existingMounts = myVault.mounts;
      return myVault.getMounts().then(function (mounts) {
        debuglog(mounts);
        existingMounts.should.be.empty;
        mounts.should.not.be.empty;
        existingMounts.should.not.contain.keys('sys/');
        mounts.should.contain.keys('sys/');
      });
    });

  });

  describe('#createMount', function () {

    it('should reject with an Error if not initialized or unsealed', function () {
      return newVault.createMount({
        id: 'other',
        body: {
          type: 'consul'
        }
      }).should.be.rejectedWith(/Vault has not been initialized/);
    });

    it('should reject with an Error if no options provided', function () {
      return myVault.createMount()
        .should.be.rejectedWith(/You must provide mount id/);
    });

    it('should reject with an Error if option id empty', function () {
      return myVault.createMount({
        id: ''
      }).should.be.rejectedWith(/You must provide mount id/);
    });

    it('should reject with an Error if option body empty', function () {
      return myVault.createMount({
        id: 'xzy',
        body: null
      }).should.be.rejectedWith(/You must provide mount details/);
    });

    it('should reject with an Error if option body without type', function () {
      return myVault.createMount({
        id: 'xzy',
        body: {}
      }).should.be.rejectedWith(/You must provide mount type/);
    });

    it('should reject with an Error if option body with empty type', function () {
      return myVault.createMount({
        id: 'xzy',
        body: {
          type: ''
        }
      }).should.be.rejectedWith(/You must provide mount type/);
    });

    it('should resolve to updated list of mounts', function () {
      var existingMounts = _.cloneDeep(myVault.mounts);
      return myVault.createMount({
        id: 'other',
        body: {
          type: 'consul'
        }
      }).then(function (mounts) {
        debuglog(mounts);
        existingMounts.should.not.be.empty;
        mounts.should.not.be.empty;
        existingMounts.should.not.contain.keys('other/');
        mounts.should.contain.keys('other/');
      });
    });

  });

  describe('#reMount', function () {

    it('should reject no options provided', function () {
      return myVault.reMount()
        .should.be.rejectedWith(/You must provide from mount/);
    });

    it('should reject empty option from', function () {
      return myVault.reMount({
        from: ''
      }).should.be.rejectedWith(/You must provide from mount/);
    });

    it('should reject no option to', function () {
      return myVault.reMount({
        from: 'xyz'
      }).should.be.rejectedWith(/You must provide to mount/);
    });

    it('should reject empty option to', function () {
      return myVault.reMount({
        from: 'xyz',
        to: ''
      }).should.be.rejectedWith(/You must provide to mount/);
    });

    it('should reject no existing from mount', function () {
      return myVault.reMount({
        from: 'xyz',
        to: 'abc'
      }).should.be.rejectedWith(/Could not find existing mount named/);
    });

    it('should reject with an Error if not initialized or unsealed', function () {
      newVault.mounts = _.cloneDeep(myVault.mounts);
      return newVault.reMount({
        from: 'other',
        to: 'sample'
      }).should.be.rejectedWith(/Vault has not been initialized/);
    });

    it('should resolve to updated list of mounts (when slash provided)', function () {
      var existingMounts = _.cloneDeep(myVault.mounts);
      return myVault.reMount({
        from: 'other/',
        to: 'samplex'
      }).then(function (mounts) {
        debuglog(mounts);
        existingMounts.should.not.be.empty;
        mounts.should.not.be.empty;
        existingMounts.should.contain.keys('other/');
        existingMounts.should.not.contain.keys('samplex/');
        mounts.should.not.contain.keys('other/');
        mounts.should.contain.keys('samplex/');
      });
    });

    it('should resolve to updated list of mounts', function () {
      var existingMounts = _.cloneDeep(myVault.mounts);
      return myVault.reMount({
        from: 'samplex',
        to: 'sample'
      }).then(function (mounts) {
        debuglog(mounts);
        existingMounts.should.not.be.empty;
        mounts.should.not.be.empty;
        existingMounts.should.contain.keys('samplex/');
        existingMounts.should.not.contain.keys('sample/');
        mounts.should.not.contain.keys('samplex/');
        mounts.should.contain.keys('sample/');
      });
    });

  });

  describe('#deleteMount', function () {

    it('should reject with an Error if not initialized or unsealed', function () {
      return newVault.deleteMount({
        id: 'sample'
      }).should.be.rejectedWith(/Vault has not been initialized/);
    });

    it('should reject if no options provided', function () {
      return myVault.deleteMount()
        .should.be.rejectedWith(/You must provide mount id/);
    });

    it('should reject if no option id provided', function () {
      return myVault.deleteMount({
        id: ''
      }).should.be.rejectedWith(/You must provide mount id/);
    });

    it('should resolve to updated instance with mount removed', function () {
      var existingMounts = _.cloneDeep(myVault.mounts);
      return myVault.deleteMount({
        id: 'sample'
      }).then(function (mounts) {
        debuglog(mounts);
        existingMounts.should.not.be.empty;
        mounts.should.not.be.empty;
        existingMounts.should.contain.keys('sample/');
        mounts.should.not.contain.keys('sample/');
        mounts.should.contain.keys('sys/');
      });
    });

  });

  after(function () {
    if (!myVault.status.sealed) {
      return helpers.resealVault(myVault);
    }
  });

});
